#include "Blender_RenderManager.h"
#include "CondorManager.h"
#include "StringManager.h"
#include "JobManager.h"
#include "ConfigureManager.h"
#include "TemplateManager.h"

using namespace std;

using namespace CloudResourceManager;

string Blender_RenderManager::GetFileExtbyFotmat(const string& fileFormat)
{
    if (fileFormat.compare("JPEG") == 0)
    {
        return "jpg";
    }
    else
    {
        if (fileFormat.compare("BMP") == 0)
        {
            return "bmp";
        }
        else
        {
            if (fileFormat.compare("PNG") == 0)
            {
                return "png";
            }
            else
            {
                return "jpg";
            }
        }
    }
}
        //Render Task Input Parameter Format: StartFrame:EndFrame:Format
bool Blender_RenderManager::ParseInputParameter(const string& inputParam, string& startFrame, string& endFrame, string& outFormat)
{
    StringVector result = StringManager::split(inputParam, " \t\n:");
    if (result.size() == 3)
    {
        startFrame = result[0];
        endFrame = result[1];
        outFormat = result[2];
        return true;
    }
    return false;
}

/// <summary>
/// Initiate Instance:For blender rendering instance, it means that it generate s3 downdload job for user content file transfer.
/// </summary>
/// <param name="TaskId">taskid of a given task output file format</param>

bool Blender_RenderManager::InitiateInstanceByTask(int taskId)
{
    StringMap jobContext;

    return GenericSubmitJob(taskId, JobTypeEnum::S3Download, jobContext);
}


bool Blender_RenderManager::SubmitRenderJob(int taskId, const string& frame, const string& format)
{
    //start to create condor sub files and submit it.
    if (frame.size()  <= 4)
    {
        StringMap jobContext;
        string strExtName = GetFileExtbyFotmat(format);
        char buffer[32];
        strcpy(buffer, "0000");
        int pos = 4 - frame.size();
        strcpy(buffer + pos, frame.c_str());

        jobContext.insert(StringStringPair("frameNumberFormat", buffer));
        jobContext.insert(StringStringPair("formatExtName", strExtName));
        jobContext.insert(StringStringPair("frame", frame));
        jobContext.insert(StringStringPair("format", format));

        return GenericSubmitJob(taskId, JobTypeEnum::BlenderRender, jobContext);
    }
    return false;
}
/// <summary>
/// Initiate Task:For blender rendering task, it means that it generate s3 upload job for user content file transfer.
/// </summary>
/// <param name="myTask">a given task</param>
bool Blender_RenderManager::InitiateTask(int taskId)
{
    StringMap jobContext;
    return GenericSubmitJob(taskId, JobTypeEnum::S3Upload, jobContext);
}

/// <summary>
/// Start a task :For blender rendering task, it means that it check the s3 upload task, if the s3 upload job is done, then change the task status as "started"
/// </summary>
/// <param name="myTask">a given task</param>
bool Blender_RenderManager::StartTask(int taskId)
{
    //check the s3upload job, if the s3upload job is completed, then start the task
    try
    {
        //find the s3upload job
        // if it is completed, then we can change the status  of the task as "start".

        //ClusterMaxDAL.JobRepository myJobRepository = new ClusterMaxDAL.JobRepository();
        //List<ClusterMaxDAL.Job> myJobs = myJobRepository.GetInitiateJobs(myTask.TaskId);
        vector<Jobs_View> myJobs = mySqlppHelper.GetInitiateJobs(taskId);
        for (int i = 0; i < myJobs.size(); i ++)
        {
            //as long as ONE succeed, we are done!
            const Jobs_View& myJob= myJobs[i];
            bool ISS3Uploaded = false;
            //ConfigLogger.Instance.LogInfo("condor_find_job.ps1: s3 upload job :" + myJob.CondorJobId);
            int jobStatus = JobManager::GetCondorJobStatus(myJob.CondorJobId);
            //ConfigLogger.Instance.LogInfo("s3 upload job condor status :" + jobStatus.ToString());
            if (jobStatus != JobStatusEnum::Unknown)
            {
                if (jobStatus == JobStatusEnum::Finish)
                {
                    ISS3Uploaded = true;
                }
                if (jobStatus != myJob.JobStatusId)
                {
                    if (!mySqlppHelper.UpdateJobStatusId(myJob.JobId, jobStatus))
                    {
                        FILE_LOG(logERROR)<<"update job status error for job id "<<myJob.JobId<<" of job status "<<jobStatus;
                    }
                }
            }

            if (!ISS3Uploaded)
            {
                return false;
            }
        }
        return true;
    }
    catch (runtime_error e)
    {
        //ConfigLogger.Instance.LogError("StartTask:" + e.Message);
        FILE_LOG(logERROR)<< "StartTask:" << e.what();
        return false;
    }
}

string Blender_RenderManager::GetResultFileName(const string& strPlatform, const StringMap& jobContext)
{
    //$s3BucketPrefix$s3Bucket/task$task_frame$frameNumberFormat.$formatExtName
    StringMap::const_iterator it;
    string strResult;
    string strTask, strFormat, strExt;

    if ((it = jobContext.find("task")) != jobContext.end())
    {
        strTask = it->second;
        if ((it = jobContext.find("frameNumberFormat")) != jobContext.end())
        {
            strFormat = it->second;
            if ((it = jobContext.find("formatExtName")) != jobContext.end())
            {
                strResult = ConfigureManager::Get(strPlatform, "AmazonS3") + "/" + ConfigureManager::Get(strPlatform, "s3Bucket") + "/task";
                strResult += strTask + "_frame" + strFormat + "." + strExt;
            }
        }
    }
    if (strResult.size() == 0)
    {
        FILE_LOG(logERROR)<< "GetResultFileName cannot find task/format/ext";
    }
    return strResult;
}


bool Blender_RenderManager::GenericSubmitJob(int nTaskId, int jobType, StringMap& jobContext)
{
    try
    {
        Tasks_View myTask;
        if (!mySqlppHelper.GetTasks_ViewById(nTaskId, myTask))
        {
            return false;
        }
        Jobs newJob;
        string strPlatform = myTask.PlatformName;
        string strTaskId = StringManager::int2string(nTaskId);

        newJob.TaskId = nTaskId;
        newJob.JobTypeId = (int)jobType;
        newJob.JobInputParameter = myTask.TaskInputParameter;

        int jobId = mySqlppHelper.SaveNewJobs(newJob);
        if (jobId == 0)
        {
            FILE_LOG(logERROR)<<"submit new job failed for taskId "<<nTaskId;
            return false;
        }
        newJob.JobId = jobId;
        // TO DO: redesign algo for finding file path!!!!
        // initialize jobContext
        //taskMainfile： 从提交的form字段renderfile获得，请参考代码文件/view/submit/index.aspx里的隐藏字段renderfile是当前文件树用户选择路径的存储变量。
        //（我们后来讨论使用用户历史文件列表，不知你是否施行，是否有建立一个userFile表？)
        //taskInputFolder：这个从系统配置文件的userBaseFolder + userName获得。（如果你有什么需要设定的参数不妨都写在一个配置文件里
        string userName = myTask.UserName;
        //int index = myTask.TaskInputFolder.find_last_of('/');
        // the actual name includes user name
        // the format is "TaskInputFolder/TaskMainFile"
        //string strFilePath = myTask.TaskMainFile.substr(index + 1 + userName.size() + 1);
        int index = myTask.TaskInputFolder.size();
        string strFilePath = myTask.TaskMainFile.substr(index + 1);
        //convert window style path to linux
        //strFilePath = strFilePath.Replace('\\', '/');
        //ConfigLogger.Instance.LogInfo("jobcontext: user=" + userName + " file=" + strFilePath);
        // only after new job is saved, can we get jobId
        string strJobId = StringManager::int2string(jobId);

        jobContext.insert(StringStringPair("job", strJobId));
        jobContext.insert(StringStringPair("task", strTaskId));
        jobContext.insert(StringStringPair("user", userName));
        jobContext.insert(StringStringPair("file", strFilePath));
        // we need to jobContext to be initialized
        FILE_LOG(logDEBUG)<<"jobContext:[job]"<<strJobId<<"[task]"<<strTaskId<<"[user]"<<userName<<"[file]"<<strFilePath;
        switch (jobType)
        {
        case JobTypeEnum::S3Upload:
            newJob.JobDesc = "Upload user content to S3 of task ";
            newJob.JobDesc += strTaskId;
            newJob.JobStatusId = (int)JobStatusEnum::New;
            break;
        case JobTypeEnum::S3Download:
            newJob.JobDesc = "Download user content from S3 of task ";
            newJob.JobDesc += strTaskId;
            newJob.JobStatusId = (int)JobStatusEnum::Initiate;
            break;
        case JobTypeEnum::BlenderRender:
            newJob.JobDesc = "render job of task ";
            newJob.JobDesc += strTaskId;
            newJob.JobStatusId = (int)JobStatusEnum::Start;
            newJob.JobInputFolder = myTask.TaskInputFolder;
            newJob.JobInputMainFile = myTask.TaskMainFile;
            newJob.JobOutputMainFile = GetResultFileName(strPlatform, jobContext);
            //ConfigLogger.Instance.LogInfo("job output file is:" + newJob.JobOutputMainFile);
            break;
        }

        //generate the s3upload sub file
        //ConfigLogger.Instance.LogInfo(newJob.JobDesc);

        string subFile = GenericGenerateJobFiles(strTaskId, strPlatform, strJobId, jobType, jobContext);
        //upload to s3
        //ConfigLogger.Instance.LogDebug("ConfigureManager.userScriptPath=" + ConfigureManager.userScriptPath);


        if (CondorManager::submit_job(subFile, newJob.CondorJobId))
        {
            if (mySqlppHelper.UpdateJobs(newJob.JobId, newJob))
            {
                return true;
            }
        }
            // we need to save the
        //ConfigLogger.Instance.LogError("condor submit cannot return job id:" + newJob.TaskId.ToString() );
        FILE_LOG(logERROR)<< "condor submit cannot return job id:" << newJob.TaskId;
        newJob.JobStatusId = (int)JobStatusEnum::Error;
        mySqlppHelper.UpdateJobs(newJob.JobId, newJob);
        return false;

    }
    catch (runtime_error e)
    {
        //ConfigLogger.Instance.LogError("InitiateTask:" + e.InnerException.ToString());
        FILE_LOG(logERROR)<< "InitiateTask:"<<e.what();
        return false;
    }
}

        // strFrame, strFormat would be null except for render jobs
string Blender_RenderManager::GenericGenerateJobFiles(const string& taskId, const string& strPlatform, const string& jobId,
        int jobType, StringMap& jobContext)
{
    string subTemplate;
    string batTempate;

    string subFile;
    string batchFile;
    string strFileNameBase = "Task" + taskId + "_Job"+ jobId;

    subFile = ConfigureManager::Get(strPlatform, "userSubmitPath");

    StringManager::validatePath(subFile);

    batchFile = ConfigureManager::Get(strPlatform, "userBatchPath");
    StringManager::validatePath(batchFile);

    subTemplate = ConfigureManager::Get(strPlatform, "userTemplatePath");
    StringManager::validatePath(subTemplate);

    batTempate = subTemplate;
    switch (jobType)
    {
    case JobTypeEnum::S3Download:
        subFile += "Download_";
        subFile += strFileNameBase;
        subFile += ".sub";

        batchFile += "Download_";
        batchFile += strFileNameBase;
        batchFile += ".bat";

        subTemplate += ConfigureManager::Get(strPlatform, "downloadSubmitTemplateName");
        batTempate = batTempate + "/" + ConfigureManager::Get(strPlatform, "downloadBatchTemplateName");
        TemplateManager::GenerateScriptFile(strPlatform, subTemplate, subFile, jobContext);
        TemplateManager::GenerateScriptFile(strPlatform, batTempate, batchFile, jobContext);
        break;
    case JobTypeEnum::S3Upload:
        subFile += "Upload_" + strFileNameBase + ".sub";
        batchFile += "Upload_" +  strFileNameBase + ".bat";
        subTemplate = subTemplate + "/" + ConfigureManager::Get(strPlatform, "uploadSubmitTemplateName");
        batTempate = batTempate + "/" + ConfigureManager::Get(strPlatform, "uploadBatchTemplateName");
        TemplateManager::GenerateScriptFile(strPlatform, subTemplate, subFile, jobContext);
        TemplateManager::GenerateScriptFile(strPlatform, batTempate, batchFile, jobContext);
        break;
    case JobTypeEnum::BlenderRender:
        subFile += "Render_" + strFileNameBase + ".sub";
        batchFile += "Render_" + strFileNameBase + ".bat";

        subTemplate = subTemplate + "/" + ConfigureManager::Get(strPlatform, "renderSubmitTemplateName");
        batTempate = batTempate + "/" + ConfigureManager::Get(strPlatform, "renderBatchTemplateName");

        TemplateManager::GenerateScriptFile(strPlatform, subTemplate, subFile, jobContext);
        TemplateManager::GenerateScriptFile(strPlatform, batTempate, batchFile, jobContext);
        break;
    }
    return subFile;
}


/// <summary>
///generate related rendering jobs by a given task
/// </summary>
/// <param name="myTask">the given task</param>
///
///
int Blender_RenderManager::GenerateRenderJobs(const Tasks& myTask)
{
    //get the start and end frames
    int iResult = 0;
    string strStartFrame;
    string strEndFrame;
    string outForamt;

    if (ParseInputParameter(myTask.TaskInputParameter, strStartFrame, strEndFrame, outForamt))
    {
        int nStartFrame;
        if (!StringManager::string2int(strStartFrame, nStartFrame))
        {
            return false;
        }
        int nEndFrame;
        StringManager::string2int(strEndFrame, nEndFrame);
        FILE_LOG(logINFO)<<"Startframe:" << strStartFrame <<" EndFrame:"<<strEndFrame<<" outformat:" <<outForamt;

        //ConfigLogger.Instance.LogStatus("Startframe:" + sframe + " EndFrame:" + eframe + " outformat:" + outforamt);
        //generate render jobs
        try
        {
            //this should be best done when web submitted....
            if (nEndFrame >= nStartFrame)
            {
                for (int i = nStartFrame; i <= nEndFrame; i++)
                {

                    //ConfigLogger.Instance.LogInfo("task input main file:" + myTask.TaskMainFile);
                    FILE_LOG(logINFO)<<"task input main file:" << myTask.TaskMainFile;
                    //submit the job to condor
                    string strTaskId = StringManager::int2string(myTask.TaskId);
                    string strFrame = StringManager::int2string(i);
                    if (SubmitRenderJob(myTask.TaskId, strFrame, outForamt))
                    {
                        iResult++;
                    }
                }
            }
        }
        catch (runtime_error e)
        {
            FILE_LOG(logERROR)<<"Error on generate render jobs:"<<e.what();
            //System.Diagnostics.StackTrace trace = new System.Diagnostics.StackTrace(e, true);
            //ConfigLogger.Instance.LogError("Error on generate render jobs:" + e.Message);
            //ConfigLogger.Instance.LogError(trace.GetFrame(0).GetMethod().Name);
            //ConfigLogger.Instance.LogError("Line: " + trace.GetFrame(0).GetFileLineNumber());

        }
    }
    //ConfigLogger.Instance.LogStatus("Finnish the generation of render jobs");
    FILE_LOG(logINFO)<<"Finnish the generation of render jobs";
    return iResult;
}
